import 'package:flutter/material.dart';
import 'package:flutterskel/provider/ProviderBuilder.dart' as arch;

import '../configuration/AppConfiguration.dart';
import '../data/sources/DataSource.dart';
import '../resources/colors/AppColors.dart';
import '../resources/dimensions/AppDimensions.dart';
import '../resources/fonts/AppFonts.dart';
import '../resources/icons/AppIcons.dart';
import '../resources/images/AppImages.dart';
import '../resources/strings/AppStrings.dart';
import '../resources/themes/AppTheme.dart';
import '../routing/page/PageRouter.dart';
import '../routing/url/UrlRouter.dart';
import '../userinformation/UserInformationRepository.dart';

class ProviderBuilder
    extends arch.ProviderBuilder<
        DataSource,
        // Routing
        UrlRouter,
        PageRouter,
        // Configuration
        AppConfiguration,
        // UserInformation
        UserInformationRepository,
        // Theme
        AppColors,
        AppDimensions,
        AppFonts,
        AppTheme,
        // Strings
        AppStrings,
        // Images and Icons
        AppImages,
        AppIcons> {


  @override
  DataSource buildDataSource() {
    return DataSource();
  }

  @override
  AppTheme buildTheme() {
    return AppTheme(
        dimensions: getDimensions(),
        colors: getColors(),
        fonts: getFonts(),
        materialTheme: ThemeData.dark()
    );
  }

  @override
  AppIcons buildIcons() {
    return AppIcons();
  }

  @override
  AppFonts buildFonts() {
    return AppFonts();
  }

  @override
  AppDimensions buildDimensions() {
    return AppDimensions();
  }

  @override
  AppColors buildColors() {
    return AppColors();
  }

  @override
  UserInformationRepository buildUserInformationRepository() {
    return UserInformationRepository();
  }

  @override
  AppStrings buildStringsResources() {
    return AppStrings();
  }

  @override
  AppImages buildImages() {
    return AppImages();
  }

  @override
  AppConfiguration buildConfiguration() {
    return AppConfiguration();
  }

  @override
  UrlRouter buildUrlRouter() {
    return UrlRouter();
  }

  @override
  PageRouter buildPageRouter() {
    return PageRouter();
  }
}