import 'package:flutter/material.dart';
import 'package:flutterskel/resources/colors/ColorResources.dart';
import 'package:flutterskel/resources/dimensions/DimensionResources.dart';

/// Base class for fonts
/// Provides basic Material Design textStyles
abstract class FontResources {

  // Colors
  ColorResources colors;

  // Dimensions
  DimensionResources dimensions;
  
  // Font Family
  String fontFamily;

  // Font Styles
  FontStyle fontStyleHeadline1;
  FontStyle fontStyleHeadline2;
  FontStyle fontStyleHeadline3;
  FontStyle fontStyleHeadline4;
  FontStyle fontStyleHeadline5;
  FontStyle fontStyleHeadline6;
  FontStyle fontStyleSectionTitle;
  FontStyle fontStyleSubtitle1;
  FontStyle fontStyleSubtitle2;
  FontStyle fontStyleBody1;
  FontStyle fontStyleBody2;
  FontStyle fontStyleButton;
  FontStyle fontStyleCaption;
  FontStyle fontStyleOverline;

  // Font Weights
  FontWeight fontWeightHeadline1;
  FontWeight fontWeightHeadline2;
  FontWeight fontWeightHeadline3;
  FontWeight fontWeightHeadline4;
  FontWeight fontWeightHeadline5;
  FontWeight fontWeightHeadline6;
  FontWeight fontWeightSectionTitle;
  FontWeight fontWeightSubtitle1;
  FontWeight fontWeightSubtitle2;
  FontWeight fontWeightBody1;
  FontWeight fontWeightBody2;
  FontWeight fontWeightButton;
  FontWeight fontWeightCaption;
  FontWeight fontWeightOverline;

  // Text Styles
  TextStyle styleHeadline1;
  TextStyle styleHeadline2;
  TextStyle styleHeadline3;
  TextStyle styleHeadline4;
  TextStyle styleHeadline5;
  TextStyle styleHeadline6;
  TextStyle styleSectionTitle;
  TextStyle styleSubtitle1;
  TextStyle styleSubtitle2;
  TextStyle styleBody1;
  TextStyle styleBody2;
  TextStyle styleButton;
  TextStyle styleCaption;
  TextStyle styleOverline;
  
  FontResources({
    // Colors
    @required this.colors,
    // Dimensions
    @required this.dimensions,

    // Font Family
    this.fontFamily,

    // Font Style
    this.fontStyleHeadline1 = FontStyle.normal,
    this.fontStyleHeadline2 = FontStyle.normal,
    this.fontStyleHeadline3 = FontStyle.normal,
    this.fontStyleHeadline4 = FontStyle.normal,
    this.fontStyleHeadline5 = FontStyle.normal,
    this.fontStyleHeadline6 = FontStyle.normal,
    this.fontStyleSectionTitle = FontStyle.normal,
    this.fontStyleSubtitle1 = FontStyle.normal,
    this.fontStyleSubtitle2 = FontStyle.normal,
    this.fontStyleBody1 = FontStyle.normal,
    this.fontStyleBody2 = FontStyle.normal,
    this.fontStyleButton = FontStyle.normal,
    this.fontStyleCaption = FontStyle.normal,
    this.fontStyleOverline = FontStyle.normal,

    // Font Weight
    this.fontWeightHeadline1 = FontWeight.w600,
    this.fontWeightHeadline2 = FontWeight.w400,
    this.fontWeightHeadline3 = FontWeight.w400,
    this.fontWeightHeadline4 = FontWeight.w400,
    this.fontWeightHeadline5 = FontWeight.w400,
    this.fontWeightHeadline6 = FontWeight.w400,
    this.fontWeightSectionTitle = FontWeight.w400,
    this.fontWeightSubtitle1 = FontWeight.w400,
    this.fontWeightSubtitle2 = FontWeight.w400,
    this.fontWeightBody1 = FontWeight.w400,
    this.fontWeightBody2 = FontWeight.w400,
    this.fontWeightButton = FontWeight.w400,
    this.fontWeightCaption = FontWeight.w400,
    this.fontWeightOverline = FontWeight.w500,
  }) {
    
    TextStyle baseStyle = TextStyle();
    if(fontFamily != null) {
      baseStyle = TextStyle(fontFamily: this.fontFamily);
    } else {
      baseStyle = TextStyle();
    }
    
    styleHeadline1 = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightHeadline1, fontStyle: fontStyleHeadline1,
        letterSpacing: dimensions.fontLetterSpacingHeadline1, fontSize: dimensions.fontSizeHeadline1);
    styleHeadline2 = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightHeadline2, fontStyle: fontStyleHeadline2,
        letterSpacing: dimensions.fontLetterSpacingHeadline2, fontSize: dimensions.fontSizeHeadline2);
    styleHeadline3 = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightHeadline3, fontStyle: fontStyleHeadline3,
        letterSpacing: dimensions.fontLetterSpacingHeadline3, fontSize: dimensions.fontSizeHeadline3);
    styleHeadline4 = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightHeadline4, fontStyle: fontStyleHeadline4,
        letterSpacing: dimensions.fontLetterSpacingHeadline4, fontSize: dimensions.fontSizeHeadline4);
    styleHeadline5 = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightHeadline5, fontStyle: fontStyleHeadline5,
        letterSpacing: dimensions.fontLetterSpacingHeadline5, fontSize: dimensions.fontSizeHeadline5);
    styleHeadline6 = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightHeadline6, fontStyle: fontStyleHeadline6,
        letterSpacing: dimensions.fontLetterSpacingHeadline6, fontSize: dimensions.fontSizeHeadline6);

    styleSectionTitle = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightSectionTitle, fontStyle: fontStyleSectionTitle,
        letterSpacing: dimensions.fontLetterSpacingSectionTitle, fontSize: dimensions.fontSizeSectionTitle);
    
    styleSubtitle1 = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightSubtitle1, fontStyle: fontStyleSubtitle1,
        letterSpacing: dimensions.fontLetterSpacingSubtitle1, fontSize: dimensions.fontSizeSubtitle1);
    styleSubtitle2 = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightSubtitle2, fontStyle: fontStyleSubtitle2,
        letterSpacing: dimensions.fontLetterSpacingSubtitle2, fontSize: dimensions.fontSizeSubtitle2);

    styleBody1 = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightBody1, fontStyle: fontStyleBody1,
        letterSpacing: dimensions.fontLetterSpacingBody1, fontSize: dimensions.fontSizeBody1);
    styleBody2 = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightBody2, fontStyle: fontStyleBody2,
        letterSpacing: dimensions.fontLetterSpacingBody2, fontSize: dimensions.fontSizeBody2);

    styleButton = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightButton, fontStyle: fontStyleButton,
        letterSpacing: dimensions.fontLetterSpacingButton, fontSize: dimensions.fontSizeButton);

    styleCaption = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightCaption, fontStyle: fontStyleCaption,
        letterSpacing: dimensions.fontLetterSpacingCaption, fontSize: dimensions.fontSizeCaption);

    styleOverline = baseStyle.copyWith(color: colors.textPrimary,
        fontWeight: fontWeightOverline, fontStyle: fontStyleOverline,
        letterSpacing: dimensions.fontLetterSpacingOverline, fontSize: dimensions.fontSizeOverline);
    

  }


}